/* Simplify psl - print only select fields of psl alignment. */
#include "common.h"
#include "linefile.h"
#include "psl.h"

void usage()
/* Explain usage and exit. */
{
errAbort(
"pslSimp - create simplified version of psl file.\n"
"usage:\n"
"    pslSimp in.psl. out.ps");
}

void pslSimp(char *inName, char *outName)
/* Simplify psl - print only select non-tab fields. */
{
struct lineFile *lf = lineFileOpen(inName, TRUE);
FILE *f = mustOpen(outName, "w");
struct psl *psl;
char *line;
int lineSize;
int totMatch = 0, totMis = 0, totIns = 0, totRepMatch = 0;

if (!lineFileNext(lf, &line, &lineSize))
    errAbort("%s is empty\n", inName);
if (startsWith(line, "psLayout version"))
   {
   int i;
   for (i=0; i<4; ++i)
       lineFileNext(lf, &line, &lineSize);
   }
else
    lineFileReuse(lf);
while ((psl = pslNext(lf)) != NULL)
    {
    fprintf(f, "%4d %2d %4d %2d %s %s %6d %6d %6d %s %6d %6d %6d\n",
      psl->match, psl->misMatch, psl->repMatch, psl->blockCount-1, psl->strand,
      psl->qName, psl->qSize, psl->qStart, psl->qEnd,
      psl->tName, psl->tSize, psl->tStart, psl->tEnd);
    totMatch += psl->match;
    totMis += psl->misMatch;
    totIns += psl->blockCount-1;
    totRepMatch += psl->repMatch;
    pslFree(&psl);
    }
fprintf(f, "Total match %d, mismatch %d, rep match %d, insert %d\n",
	totMatch, totMis, totRepMatch, totIns);
fclose(f);
lineFileClose(&lf);
}

int main(int argc, char *argv[])
/* Process command line. */
{
if (argc != 3)
    usage();
pslSimp(argv[1], argv[2]);
}
