/*****************************************************************************
 * Copyright (C) 2000 Jim Kent.  This source code may be freely used         *
 * for personal, academic, and non-profit purposes.  Commercial use          *
 * permitted only by explicit agreement with Jim Kent (jim_kent@pacbell.net) *
 *****************************************************************************/
/* Nib - nibble (4 bit) representation of nucleotide sequences. */
#include "common.h"
#include "dnautil.h"
#include "dnaseq.h"
#include "nib.h"
#include "sig.h"

void nibOpenVerify(char *fileName, FILE **retFile, int *retSize)
/* Open file and verify it's in good nibble format. */
{
bits32 size;
bits32 sig;
FILE *f = mustOpen(fileName, "rb");

mustReadOne(f, sig);
mustReadOne(f, size);
if (sig != nibSig)
    {
    sig = byteSwap32(sig);
    size = byteSwap32(size);
    if (sig != nibSig)
	errAbort("%s is not a good .nib file.",  fileName);
    }
*retSize = size;
*retFile = f;
}

struct dnaSeq *nibLoadPart(char *fileName, int start, int size)
/* Load part of an .nib file. */
{
FILE *f;
int seqSize;
int end;
DNA *d;
int bVal;
struct dnaSeq *seq;
int bytePos, byteSize;
char nameBuf[512];

nibOpenVerify(fileName, &f, &seqSize);
assert(start >= 0);
assert(size >= 0);
end = start+size;
if (end >= seqSize)
    errAbort("nibLoadPart past end of file (%d %d)", end, seqSize);

AllocVar(seq);
seq->size = size;
sprintf(nameBuf, "%s:%d-%d", fileName, start, start+size);
seq->name = cloneString(nameBuf);
seq->dna = d = needLargeMem(size+1);
bytePos = (start>>1);
fseek(f, bytePos, SEEK_CUR);
if (start & 1)
    {
    bVal = getc(f);
    if (bVal < 0)
	errAbort("Read error in %s", fileName);
    *d++ = valToNt[(bVal>>4)];
    size -= 1;
    }
byteSize = (size>>1);
while (--byteSize >= 0)
    {
    bVal = getc(f);
    if (bVal < 0)
	errAbort("Read error in %s", fileName);
    d[0] = valToNt[(bVal>>4)];
    d[1] = valToNt[(bVal&0xf)];
    d += 2;
    }
if (size&1)
    {
    bVal = getc(f);
    if (bVal < 0)
	errAbort("Read error in %s", fileName);
    *d++ = valToNt[(bVal>>4)];
    }
*d = 0;
fclose(f);
return seq;
}
